@@ -2,16 +2,23 @@
 
 from typing import Optional
 
-from .base import BaseParser, ImportInfo, FunctionInfo, ClassInfo, ParseResult
+from .base import (
+    BaseParser, ImportInfo, FunctionInfo, ClassInfo, ParseResult, ParseError, ParseStats,
+    DecoratorInfo, AttributeInfo, GlobalVariableInfo, StructFieldInfo, InterfaceMethodInfo,
+    StructInfo, InterfaceInfo, EnhancedFunctionInfo, EnhancedClassInfo, EnhancedParseResult,
+    ParseErrorHandler
+)
 from .python_parser import PythonParser
 from .js_parser import JsParser
 from .vue_parser import VueParser
+from .go_parser import GoParser
 
 
 # Singleton parser instances
 _python_parser = PythonParser()
 _js_parser = JsParser()
 _vue_parser = VueParser()
+_go_parser = GoParser()
 
 # Map file extensions to parsers
 _PARSER_MAP: dict[str, BaseParser] = {
@@ -24,6 +31,7 @@ _PARSER_MAP: dict[str, BaseParser] = {
     ".ts": _js_parser,
     ".tsx": _js_parser,
     ".vue": _vue_parser,
+    ".go": _go_parser,
 }
 
 
@@ -45,8 +53,22 @@ __all__ = [
     "FunctionInfo",
     "ClassInfo",
     "ParseResult",
+    "ParseError",
+    "ParseStats",
+    "ParseErrorHandler",
+    "DecoratorInfo",
+    "AttributeInfo",
+    "GlobalVariableInfo",
+    "StructFieldInfo",
+    "InterfaceMethodInfo",
+    "StructInfo",
+    "InterfaceInfo",
+    "EnhancedFunctionInfo",
+    "EnhancedClassInfo",
+    "EnhancedParseResult",
     "PythonParser",
     "JsParser",
     "VueParser",
+    "GoParser",
     "get_parser",
 ]
