@@ -5,6 +5,7 @@ that exposes code knowledge graph tools for AI integration.
 """
 
 import json
+from enum import Enum
 from pathlib import Path
 from typing import Any, Optional
 
@@ -21,7 +22,34 @@ from core.services import (
     create_llm_provider,
     create_embedding_provider,
     create_summarizer,
+    # New services for enhanced features
+    SymbolSearchService,
+    SymbolType,
+    SkeletonExtractor,
+    SkeletonMode,
+    EntryPointDetector,
+    EntryPointType,
 )
+from core.services.call_chain import CallChainService
+
+
+class MCPErrorCode(Enum):
+    """MCP error codes for structured error responses.
+    
+    Provides standardized error codes for all MCP tool responses.
+    
+    **Validates: Requirement 6.6** - Structured error responses
+    """
+    NOT_FOUND = "NOT_FOUND"
+    INVALID_PARAMS = "INVALID_PARAMS"
+    PARSE_ERROR = "PARSE_ERROR"
+    INTERNAL_ERROR = "INTERNAL_ERROR"
+    NOT_SCANNED = "NOT_SCANNED"
+    LIMIT_EXCEEDED = "LIMIT_EXCEEDED"
+    UNKNOWN_TOOL = "UNKNOWN_TOOL"
+    LLM_NOT_CONFIGURED = "LLM_NOT_CONFIGURED"
+    EMBEDDING_NOT_CONFIGURED = "EMBEDDING_NOT_CONFIGURED"
+    SEARCH_ERROR = "SEARCH_ERROR"
 
 
 class MCPServer:
@@ -71,6 +99,10 @@ class MCPServer:
             llm_config=llm_config,
             embedding_config=embedding_config
         )
+        
+        # Initialize new services for enhanced features
+        self.symbol_search_service = SymbolSearchService(self.storage)
+        self.call_chain_service = CallChainService(self.storage)
 
     @classmethod
     def from_config(
@@ -231,7 +263,7 @@ class MCPServer:
             },
             {
                 "name": "get_related_code_context",
-                "description": "获取文件的关联代码上下文(Repo Map)，包含目标文件及其依赖文件的签名和摘要 | Get related code context (Repo Map) with signatures and summaries",
+                "description": "获取文件的关联代码上下文(Repo Map)，支持骨架模式以减少Token消耗 | Get related code context (Repo Map) with skeleton mode support for reduced token usage",
                 "inputSchema": {
                     "type": "object",
                     "properties": {
@@ -249,6 +281,12 @@ class MCPServer:
                             "minimum": 1,
                             "maximum": 3,
                             "description": "依赖跳数 | Dependency hops"
+                        },
+                        "mode": {
+                            "type": "string",
+                            "enum": ["full", "skeleton", "signature_only"],
+                            "default": "skeleton",
+                            "description": "内容模式：full（完整内容）、skeleton（骨架模式，保留签名和文档）、signature_only（仅签名）| Content mode: full (complete content), skeleton (signatures and docs), signature_only (signatures only)"
                         }
                     },
                     "required": ["project_path", "file_path"]
@@ -308,6 +346,115 @@ class MCPServer:
                     "properties": {},
                     "required": []
                 }
+            },
+            # New tools for enhanced features (Task 8)
+            {
+                "name": "trace_call_chain",
+                "description": "追踪函数调用链，支持向上追溯（谁调用了我）和向下追踪（我调用了谁）| Trace function call chain, supports upstream (who calls me) and downstream (who do I call) tracing",
+                "inputSchema": {
+                    "type": "object",
+                    "properties": {
+                        "project_path": {
+                            "type": "string",
+                            "description": "项目路径 | Project path"
+                        },
+                        "start_symbol": {
+                            "type": "string",
+                            "description": "起始函数名 | Starting function name"
+                        },
+                        "direction": {
+                            "type": "string",
+                            "enum": ["upstream", "downstream"],
+                            "default": "downstream",
+                            "description": "追踪方向：upstream（向上追溯调用者）或 downstream（向下追踪被调用者）| Trace direction: upstream (trace callers) or downstream (trace callees)"
+                        },
+                        "depth": {
+                            "type": "integer",
+                            "default": 5,
+                            "minimum": 1,
+                            "maximum": 10,
+                            "description": "最大追踪深度 | Maximum trace depth"
+                        },
+                        "limit_per_level": {
+                            "type": "integer",
+                            "default": 20,
+                            "minimum": 1,
+                            "maximum": 100,
+                            "description": "每层最大节点数 | Maximum nodes per depth level"
+                        },
+                        "prune_standard_libs": {
+                            "type": "boolean",
+                            "default": True,
+                            "description": "是否剪枝标准库和第三方库调用 | Whether to prune standard library and third-party calls"
+                        }
+                    },
+                    "required": ["project_path", "start_symbol"]
+                }
+            },
+            {
+                "name": "find_entry_points",
+                "description": "查找项目入口点，包括HTTP路由、main函数、数据库模型和CLI命令 | Find project entry points including HTTP routes, main functions, database models, and CLI commands",
+                "inputSchema": {
+                    "type": "object",
+                    "properties": {
+                        "project_path": {
+                            "type": "string",
+                            "description": "项目路径 | Project path"
+                        },
+                        "types": {
+                            "type": "array",
+                            "items": {
+                                "type": "string",
+                                "enum": ["http_routes", "main_entry", "database_models", "cli_commands"]
+                            },
+                            "description": "要查找的入口点类型（可选，不指定则查找所有类型）| Entry point types to find (optional, finds all if not specified)"
+                        }
+                    },
+                    "required": ["project_path"]
+                }
+            },
+            {
+                "name": "symbol_search",
+                "description": "基于AST的精确符号搜索，支持类型过滤和前缀匹配 | AST-based precise symbol search with type filtering and prefix matching",
+                "inputSchema": {
+                    "type": "object",
+                    "properties": {
+                        "project_path": {
+                            "type": "string",
+                            "description": "项目路径 | Project path"
+                        },
+                        "query": {
+                            "type": "string",
+                            "description": "搜索查询（符号名称）| Search query (symbol name)"
+                        },
+                        "symbol_types": {
+                            "type": "array",
+                            "items": {
+                                "type": "string",
+                                "enum": ["function", "class", "method", "variable", "struct", "interface"]
+                            },
+                            "description": "符号类型过滤（可选）| Symbol type filter (optional)"
+                        },
+                        "case_sensitive": {
+                            "type": "boolean",
+                            "default": False,
+                            "description": "是否大小写敏感 | Whether to match case"
+                        },
+                        "prefix_match": {
+                            "type": "boolean",
+                            "default": True,
+                            "description": "是否启用前缀匹配 | Whether to enable prefix matching"
+                        },
+                        "limit": {
+                            "type": "integer",
+                            "default": 50,
+                            "minimum": 1,
+                            "maximum": 200,
+                            "description": "返回结果数量限制 | Limit of results"
+                        }
+                    },
+                    "required": ["project_path", "query"]
+                }
             }
         ]
 
@@ -336,6 +483,10 @@ class MCPServer:
             "generate_summaries": self._handle_generate_summaries,
             "semantic_search": self._handle_semantic_search,
             "get_summary_status": self._handle_get_summary_status,
+            # New handlers for enhanced features (Task 8)
+            "trace_call_chain": self._handle_trace_call_chain,
+            "find_entry_points": self._handle_find_entry_points,
+            "symbol_search": self._handle_symbol_search,
         }
 
         handler = handlers.get(name)
@@ -532,31 +683,44 @@ class MCPServer:
             )
 
     async def _handle_get_related_code_context(self, args: dict) -> dict:
-        """Handle get_related_code_context tool call."""
+        """Handle get_related_code_context tool call.
+        
+        **Validates: Requirement 6.2** - mode parameter support for skeleton view
+        """
         project_path = args.get("project_path")
         file_path = args.get("file_path")
         hops = args.get("hops", 1)
+        mode = args.get("mode", "skeleton")  # Default to skeleton mode
 
         if not project_path:
             raise ValueError("Missing required parameter: project_path")
         if not file_path:
             raise ValueError("Missing required parameter: file_path")
 
+        # Validate mode parameter
+        valid_modes = ["full", "skeleton", "signature_only"]
+        if mode not in valid_modes:
+            raise ValueError(f"Invalid mode: {mode}. Must be one of: {valid_modes}")
+
         project_root = Path(project_path)
         if not project_root.exists():
             return self._error_response(
                 "Project not found",
-                "NOT_FOUND",
+                MCPErrorCode.NOT_FOUND.value,
                 f"Path does not exist: {project_path}"
             )
 
-        # Create service for this project
+        # Create service for this project with skeleton extractor
         context_service = RelatedContextService(self.storage, project_root)
+        
+        # Convert mode string to SkeletonMode enum
+        skeleton_mode = SkeletonMode(mode)
 
         result = context_service.get_related_context(
             str(project_root.resolve()),
             file_path,
-            hops=hops
+            hops=hops,
+            mode=skeleton_mode
         )
 
         return context_service.to_dict(result)
@@ -718,6 +882,257 @@ class MCPServer:
             "message_zh": "LLM和嵌入模型状态"
         }
 
+    # =========================================================================
+    # New handlers for enhanced features (Task 8)
+    # =========================================================================
+
+    async def _handle_trace_call_chain(self, args: dict) -> dict:
+        """Handle trace_call_chain tool call.
+        
+        Traces function call chains using recursive CTE queries.
+        
+        **Validates: Requirement 6.1** - trace_call_chain tool
+        """
+        project_path = args.get("project_path")
+        start_symbol = args.get("start_symbol")
+        direction = args.get("direction", "downstream")
+        depth = args.get("depth", 5)
+        limit_per_level = args.get("limit_per_level", 20)
+        prune_standard_libs = args.get("prune_standard_libs", True)
+
+        if not project_path:
+            raise ValueError("Missing required parameter: project_path")
+        if not start_symbol:
+            raise ValueError("Missing required parameter: start_symbol")
+
+        # Validate direction
+        if direction not in ("upstream", "downstream"):
+            raise ValueError(f"Invalid direction: {direction}. Must be 'upstream' or 'downstream'")
+
+        project_root = Path(project_path)
+        if not project_root.exists():
+            return self._error_response(
+                "Project not found",
+                MCPErrorCode.NOT_FOUND.value,
+                f"Path does not exist: {project_path}"
+            )
+
+        # Check if project is scanned
+        project = self.storage.get_project(str(project_root.resolve()))
+        if not project:
+            return self._error_response(
+                "Project not scanned",
+                MCPErrorCode.NOT_SCANNED.value,
+                "Please scan the project first using scan_project | 请先使用scan_project扫描项目"
+            )
+
+        # Trace call chain
+        result = self.call_chain_service.trace_call_chain(
+            project_path=str(project_root.resolve()),
+            start_symbol=start_symbol,
+            direction=direction,
+            depth=depth,
+            limit_per_level=limit_per_level,
+            prune_standard_libs=prune_standard_libs
+        )
+
+        # Convert result to dict with relative paths
+        chains_data = []
+        for chain in result.chains:
+            chain_nodes = []
+            for node in chain:
+                chain_nodes.append({
+                    "function_name": node.function_name,
+                    "file_path": node.file_path,  # Already relative
+                    "line_number": node.line_number,
+                    "container_name": node.container_name,
+                    "call_type": node.call_type,
+                    "is_cycle": node.is_cycle,
+                    "depth": node.depth
+                })
+            chains_data.append(chain_nodes)
+
+        return {
+            "success": True,
+            "start_symbol": result.start_symbol,
+            "direction": result.direction,
+            "depth": result.depth,
+            "chains": chains_data,
+            "has_cycle": result.has_cycle,
+            "truncated": result.truncated,
+            "message": result.message
+        }
+
+    async def _handle_find_entry_points(self, args: dict) -> dict:
+        """Handle find_entry_points tool call.
+        
+        Finds project entry points using AST-based detection.
+        
+        **Validates: Requirement 6.3** - find_entry_points tool
+        """
+        project_path = args.get("project_path")
+        types_arg = args.get("types")
+
+        if not project_path:
+            raise ValueError("Missing required parameter: project_path")
+
+        project_root = Path(project_path)
+        if not project_root.exists():
+            return self._error_response(
+                "Project not found",
+                MCPErrorCode.NOT_FOUND.value,
+                f"Path does not exist: {project_path}"
+            )
+
+        # Check if project is scanned
+        project = self.storage.get_project(str(project_root.resolve()))
+        if not project:
+            return self._error_response(
+                "Project not scanned",
+                MCPErrorCode.NOT_SCANNED.value,
+                "Please scan the project first using scan_project | 请先使用scan_project扫描项目"
+            )
+
+        # Convert type strings to EntryPointType enum
+        entry_point_types = None
+        if types_arg:
+            try:
+                entry_point_types = [EntryPointType(t) for t in types_arg]
+            except ValueError as e:
+                raise ValueError(f"Invalid entry point type: {e}")
+
+        # Create detector and find entry points
+        detector = EntryPointDetector(
+            storage=self.storage,
+            project_root=project_root
+        )
+        
+        result = detector.find_entry_points(
+            project_path=str(project_root.resolve()),
+            types=entry_point_types
+        )
+
+        # Convert result to dict with relative paths
+        return {
+            "success": True,
+            "project_path": result.project_path,
+            "http_routes": [
+                {
+                    "file_path": r.file_path,  # Already relative
+                    "line_number": r.line_number,
+                    "route_path": r.route_path,
+                    "http_method": r.http_method,
+                    "handler_name": r.handler_name,
+                    "framework": r.framework
+                }
+                for r in result.http_routes
+            ],
+            "main_entries": [
+                {
+                    "file_path": e.file_path,
+                    "line_number": e.line_number,
+                    "entry_type": e.entry_type
+                }
+                for e in result.main_entries
+            ],
+            "database_models": [
+                {
+                    "file_path": m.file_path,
+                    "line_number": m.line_number,
+                    "model_name": m.model_name,
+                    "framework": m.framework,
+                    "table_name": m.table_name
+                }
+                for m in result.database_models
+            ],
+            "cli_commands": [
+                {
+                    "file_path": c.file_path,
+                    "line_number": c.line_number,
+                    "command_name": c.command_name,
+                    "framework": c.framework
+                }
+                for c in result.cli_commands
+            ],
+            "message": result.message
+        }
+
+    async def _handle_symbol_search(self, args: dict) -> dict:
+        """Handle symbol_search tool call.
+        
+        Performs AST-based symbol search with type filtering and prefix matching.
+        
+        **Validates: Requirement 6.4** - symbol_search tool
+        """
+        project_path = args.get("project_path")
+        query = args.get("query")
+        symbol_types_arg = args.get("symbol_types")
+        case_sensitive = args.get("case_sensitive", False)
+        prefix_match = args.get("prefix_match", True)
+        limit = args.get("limit", 50)
+
+        if not project_path:
+            raise ValueError("Missing required parameter: project_path")
+        if not query:
+            raise ValueError("Missing required parameter: query")
+
+        project_root = Path(project_path)
+        if not project_root.exists():
+            return self._error_response(
+                "Project not found",
+                MCPErrorCode.NOT_FOUND.value,
+                f"Path does not exist: {project_path}"
+            )
+
+        # Check if project is scanned
+        project = self.storage.get_project(str(project_root.resolve()))
+        if not project:
+            return self._error_response(
+                "Project not scanned",
+                MCPErrorCode.NOT_SCANNED.value,
+                "Please scan the project first using scan_project | 请先使用scan_project扫描项目"
+            )
+
+        # Convert symbol type strings to SymbolType enum
+        symbol_types = None
+        if symbol_types_arg:
+            try:
+                symbol_types = [SymbolType(t) for t in symbol_types_arg]
+            except ValueError as e:
+                raise ValueError(f"Invalid symbol type: {e}")
+
+        # Perform search
+        result = self.symbol_search_service.search(
+            project_path=str(project_root.resolve()),
+            query=query,
+            symbol_types=symbol_types,
+            case_sensitive=case_sensitive,
+            prefix_match=prefix_match,
+            limit=limit
+        )
+
+        # Convert result to dict with relative paths
+        return {
+            "success": True,
+            "query": result.query,
+            "results": [
+                {
+                    "name": r.name,
+                    "symbol_type": r.symbol_type.value,
+                    "file_path": r.file_path,  # Already relative
+                    "line_number": r.line_number,
+                    "signature": r.signature,
+                    "docstring": r.docstring,
+                    "container_name": r.container_name,
+                    "is_exported": r.is_exported,
+                    "match_score": r.match_score
+                }
+                for r in result.results
+            ],
+            "total_count": result.total_count,
+            "truncated": result.truncated
+        }
+
     def _error_response(
         self,
         error: str,
@@ -725,6 +1140,8 @@ class MCPServer:
         details: Optional[str] = None
     ) -> dict:
         """Create an error response.
+        
+        **Validates: Requirement 6.6** - Structured error responses
 
         Args:
             error: Error message
